# Generated by Django 5.0.7 on 2025-01-31 03:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("projects", "0003_project_project_manager_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="project",
            name="project_manager",
        ),
    ]
